/*
 * FILE:
 *   xna_gfx_context.hpp
 *
 * PURPOSE:
 *   Implement GfxContext for Mips/XNA
 *
 * AUTHOR:
 *   Stephen Thompson <stephen@solarflare.org.uk>
 *
 * CREATED:
 *   19-Jun-2011
 *   
 * COPYRIGHT:
 *   Copyright (C) Stephen Thompson, 2011.
 *
 *   This file is part of Mips2cs. Mips2cs is distributed under the terms
 *   of the Boost Software License, Version 1.0, the text of which
 *   appears below.
 *
 *   Boost Software License - Version 1.0 - August 17th, 2003
 *   
 *   Permission is hereby granted, free of charge, to any person or organization
 *   obtaining a copy of the software and accompanying documentation covered by
 *   this license (the "Software") to use, reproduce, display, distribute,
 *   execute, and transmit the Software, and to prepare derivative works of the
 *   Software, and to permit third-parties to whom the Software is furnished to
 *   do so, all subject to the following:
 *
 *   The copyright notices in the Software and this entire statement, including
 *   the above license grant, this restriction and the following disclaimer,
 *   must be included in all copies of the Software, in whole or in part, and
 *   all derivative works of the Software, unless such copies or derivative
 *   works are solely in the form of machine-executable object code generated by
 *   a source language processor.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 *   FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 * 
 */

#ifndef XNA_GFX_CONTEXT_HPP
#define XNA_GFX_CONTEXT_HPP

#include "gfx/gfx_context.hpp"
#include "gfx/rectangle.hpp"

namespace Coercri {

    class XNAGfxContext : public GfxContext {
    public:
        virtual ~XNAGfxContext();

        virtual void setClipRectangle(const Rectangle &rect);
        virtual void clearClipRectangle();
        virtual Rectangle getClipRectangle() const;

        virtual int getWidth() const;
        virtual int getHeight() const;
        
        virtual void clearScreen(Color col);
        virtual void plotPixel(int x, int y, Color col);
        virtual void drawGraphic(int x, int y, const Graphic &graphic);
        virtual void drawText(int x, int y, const Font &font, const std::string &text, Color col, bool antialias);

        virtual void drawLine(int x1, int y1, int x2, int y2, Color col);
        virtual void fillRectangle(const Rectangle &rect, Color col);
    };
}

#endif
