# FILE:
#   asm_test.s
#
# PURPOSE:	
#   Assembler level test case for Mips2cs. Purpose is to test all Mips opcodes
#   to check that each one is being translated correctly.
#
# AUTHOR:
#   Stephen Thompson <stephen@solarflare.org.uk>
#
# CREATED:
#   23-Oct-2010
#
# COPYRIGHT:
#   Copyright (C) Stephen Thompson, 2010 - 2011.
#
#   This file is part of Mips2cs. Mips2cs is distributed under the terms
#   of the Boost Software License, Version 1.0, the text of which
#   appears below.
#
#   Boost Software License - Version 1.0 - August 17th, 2003
#   
#   Permission is hereby granted, free of charge, to any person or organization
#   obtaining a copy of the software and accompanying documentation covered by
#   this license (the "Software") to use, reproduce, display, distribute,
#   execute, and transmit the Software, and to prepare derivative works of the
#   Software, and to permit third-parties to whom the Software is furnished to
#   do so, all subject to the following:
#
#   The copyright notices in the Software and this entire statement, including
#   the above license grant, this restriction and the following disclaimer,
#   must be included in all copies of the Software, in whole or in part, and
#   all derivative works of the Software, unless such copies or derivative
#   works are solely in the form of machine-executable object code generated by
#   a source language processor.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
#   SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
#   FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
#   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.


        .set noat

	.text


dummy_zero_return:
	li	$a1, 0
	jr	$ra
dummy_one_return:
	li	$a1, 1
	jr 	$ra
	

	# The test cases.
	# Each test case saves its result into $a1 then returns.
	# $a0 contains a pointer to the test name and should not be changed.
	# All other registers can be modified by the tests.

test_add:
	li	$s1, 4
	li	$s2, 7
	add	$a1, $s1, $s2   # expected result: 11
	jr	$ra

test_addi:
	li	$a1, 30
	addi	$a1, 12		# expected result: 42
	jr 	$ra

test_addiu:
	li	$a1, 30
	addiu	$a1, 12
	jr 	$ra		# expected result: 42

test_addiu_neg:
        li      $a1, 40
        addiu   $a1, -10        # expected result: 30 (immediate should be sign-extended, not zero-extended)
        jr      $ra
        
test_addu:
	li	$k0, 10
	li	$k1, 15
	addu	$a1, $k0, $k1	# expected result: 25
	jr	$ra

test_and:
        li      $t0, 0xf0f0
        li      $t1, 0xff0f
        and     $a1, $t0, $t1   # expected result: 61440 ($f000)
	jr 	$ra

test_andi:
	li	$t0, 3
	andi	$a1, $t0, 1	# expected result: 1
	jr 	$ra

test_andi_neg:
        lui     $t0, 0xf0f0
        andi    $a1, $t0, 0xf000  # expected result: 0 (immediate should be zero-extended, not sign-extended)
        jr      $ra
        
test_beq_false:
	li	$k0, 0
	li	$k1, 1
	beq	$k0, $k1, dummy_one_return
	j	dummy_zero_return	# expected result: 0

test_beq_true:
	li	$gp, 123
	li	$v1, 123
	beq	$gp, $v1, dummy_one_return
	j	dummy_zero_return	# expected result: 1

test_bgez_false:
	li	$gp, -1
	bgez	$gp, dummy_one_return
	j	dummy_zero_return	# expected result: 0

test_bgez_true:
	li	$gp, 1
	bgez	$gp, dummy_one_return
	j	dummy_zero_return	# expected result: 1

test_bgezal_false:
	li	$s0, -1
	li	$a1, 0
        move    $t0, $ra                # ra corrupted by bgezal, so we must save
	bgezal	$s0, dummy_one_return
	jr	$t0			# jump to saved ra. expected result: 0
	
test_bgezal_true:
	addiu	$sp, $sp, -4
	sw	$ra, ($sp)
	li	$gp, 1
	li	$a1, 0
	bgezal	$gp, dummy_one_return
	lw	$ra, ($sp)
	addiu	$sp, $sp, 4
	jr	$ra			# expected result: 1

test_bgezal_ra:
        move    $t0, $ra                # save ra
        li      $t1, -1                 # branch test value (NOT gtr or equal to zero)
        li      $ra, 0                  # set ra to zero
        bgezal  $t1, test_bgezal_ra     # branch not taken. but ra should still be overwritten.
        sltiu   $a1, $ra, 1             # Checks if $ra is still zero. Expected result: 0.
        jr      $t0                     # Jump to the saved ra.
        
test_bgtz_false:
	li	$k0, -203
	bgtz	$k0, dummy_one_return
	j	dummy_zero_return	# expected result: 0
	
test_bgtz_true:
	li	$gp, 324
	bgtz	$gp, dummy_one_return
	j	dummy_zero_return	# expected result: 1

test_blez_false:
	li	$gp, 122
	blez	$gp, dummy_one_return
	j	dummy_zero_return	# expected result: 0

test_blez_true:
	li	$gp, 0
	blez	$gp, dummy_one_return	# expected result: 1
	j	dummy_zero_return

test_bltz_false:
	li	$s7, 0
	bltz	$s7, dummy_one_return
	j	dummy_zero_return	# expected result: 0

test_bltz_true:
	li	$t9, -93
	bltz	$t9, dummy_one_return	# expected result: 1
	j	dummy_zero_return

test_bltzal_false:
	li	$k0, 12
	li	$a1, 0
        move    $t0, $ra                # ra corrupted by bltzal, so we must save
	bltzal	$k0, dummy_one_return
	jr	$t0			# jump to saved ra. expected result: 0
	
test_bltzal_true:
	addiu	$sp, $sp, -4
	sw	$ra, ($sp)
	li	$k0, -12
	li	$a1, 0
	bltzal	$k0, dummy_one_return
	lw	$ra, ($sp)
	addiu	$sp, $sp, 4	
	jr	$ra			# expected result: 1

test_bltzal_ra:
        move    $t0, $ra                # save ra
        li      $t1, 0                  # branch test value (NOT less than zero)
        li      $ra, 0                  # set ra to zero
        bltzal  $t1, test_bltzal_ra     # branch not taken, but ra should still be overwritten
        sltiu   $a1, $ra, 1             # checks if ra is still zero. expected result: 0.
        jr      $t0                     # jump to the saved ra.
        
test_bne_false:
	li	$k0, 0x12345678
	li	$k1, 0x12345678
	bne	$k0, $k1, dummy_one_return
	j	dummy_zero_return	# expected result: 0

test_bne_true:
	li	$k0, 1
	li	$s3, 2
	bne	$k0, $s3, dummy_one_return
	j	dummy_zero_return	# expected result: 1

test_delay:
        .set    noreorder
        li      $t0, 1                       # set $t0 and $t1 unequal.
        li      $t1, 2
        move    $s0, $ra                     # save $ra
        move    $s1, $a0                     # save $a0
        li      $a1, 0                       # set $a1 (length) to 0 - means jal 16 will do nothing.
        jal     16                           # prevents Mips2cs optimizing away the comparison.
        nop
        move    $ra, $s0                     # restore $ra
        move    $a0, $s1                     # restore $a0
        bne     $t0, $t1, dummy_one_return   # expected to be taken. (Expected test result is therefore 1.)
        addiu   $t1, $zero, 1                # delay slot. if done naively, will force prev branch not to be taken
        j       dummy_zero_return            # NOT expected to reach here
        nop
        .set    reorder
        
test_div_hi:
	li	$t0, -37
	li	$t1, 10
	div	$zero, $t0, $t1
	mfhi	$a1
	jr	$ra		# expected result: -7 (remainder)

test_div_lo:
	li	$t0, -37
	li	$t1, 10
	div	$zero, $t0, $t1
	mflo	$a1
	jr	$ra 		# expected result: -3 (quotient)

test_div_2hi:
	li	$t0, 42
	li	$t1, 10
	div	$zero, $t0, $t1
	mfhi	$a1
	jr	$ra		# expected: 2 (remainder)

test_div_2lo:
	li	$t0, 42
	li	$t1, 10
	div 	$zero, $t0, $t1
	mflo	$a1		# expected: 4 (quotient)
	jr	$ra

test_divu_hi:
	li	$t0, 0xffffffff
	li	$t1, 16
	divu	$zero, $t0, $t1
	mfhi	$a1
	jr	$ra		# expected: 15 (remainder)

test_divu_lo:
	li	$t0, 0xffffffff
	li	$t1, 16
	divu	$zero, $t0, $t1
	mflo	$a1		# expected: 268435455 (quotient)
	jr	$ra

test_divu_2hi:
	li	$t0, 61
	li	$t1, 6
	divu	$zero, $t0, $t1
	mfhi	$a1
	jr	$ra		# expected: 1 (remainder)

test_divu_2lo:
	li	$t0, 61
	li	$t1, 6
	divu	$zero, $t0, $t1	# expected: 10 (quotient)
	mflo	$a1
	jr	$ra

test_sb_lb:
	li	$t0, 0x12345678
	sb	$t0, -1($sp)
	lb	$a1, -1($sp)	# expected result: 120 (0x78)
	jr	$ra

test_sb_lb_2:
	li	$t0, 0x123456ff
	sb	$t0, -1($sp)
	lb	$a1, -1($sp)	# expected result: -1 (0xffffffff)
	jr	$ra

test_sb_lbu:
	li	$t0, 0x123456ff
	sb	$t0, -1($sp)
	lbu	$a1, -1($sp)	# expected result: 255 (0xff)
	jr	$ra
	
test_sh_lh:
	li	$t0, 0x12345678
	sh	$t0, -4($sp)
	lh	$a1, -4($sp)	# expected result: 22136 (0x5678)
	jr	$ra

test_sh_lh_2:
	li	$t0, 0x1234ffff
	sh	$t0, -4($sp)
	lh	$a1, -4($sp)	# expected result: -1 (0xffffffff)
	jr	$ra

test_sh_lhu:
	li	$t0, 0x1234ffff
	sh	$t0, -4($sp)
	lhu	$a1, -4($sp)	# expected result: 65535 (0xffff)
	jr	$ra
	
test_lui:
	lui	$a1, 0x1234
	jr	$ra		# expected result: 305397760 (0x12340000)

test_sw_lw:
	li	$t0, 0x12345678
	sw	$t0, -8($sp)
	lw	$a1, -8($sp)
	jr	$ra		# expected result: 305419896 (0x12345678)

test_mult_hi:
	li	$t0, 12345678
	li	$t1, 23456789
	mult	$t0, $t1
	mfhi	$a1
	jr	$ra		# expected result: 67425

test_mult_lo:	
	li	$t0, 12345678
	li	$t1, 23456789
	mult	$t0, $t1
	mflo	$a1
	jr	$ra		# expected result: 1793975142

test_mult_2hi:
	li	$t0, -12345678
	li	$t1, -23456789
	mult	$t0, $t1
	mfhi	$a1
	jr	$ra		# expected result: 67425

test_mult_2lo:	
	li	$t0, -12345678
	li	$t1, -23456789
	mult	$t0, $t1
	mflo	$a1
	jr	$ra		# expected result: 1793975142

test_multu_hi:
	li	$t0, 12345678
	li	$t1, 23456789
	multu	$t0, $t1
	mfhi	$a1
	jr	$ra		# expected result: 67425

test_multu_lo:	
	li	$t0, 12345678
	li	$t1, 23456789
	multu	$t0, $t1
	mflo	$a1
	jr	$ra		# expected result: 1793975142

test_multu_2hi:
	li	$t0, -12345678
	li	$t1, -23456789
	multu	$t0, $t1
	mfhi	$a1
	jr	$ra		# expected result: -35735042

test_multu_2lo:	
	li	$t0, -12345678
	li	$t1, -23456789
	multu	$t0, $t1
	mflo	$a1
	jr	$ra		# expected result: 1793975142

test_or:
	li	$t0, 0xf0f0
	li	$t1, 0xff00
	or	$a1, $t0, $t1	# expected: 0xfff0 = 65520
	jr 	$ra

test_ori:
	li	$t0, 0xff00
	ori	$a1, $t0, 0xf0ff
	jr	$ra		# expected: 65535

test_sll:
	li  	$t0, 2
	sll	$a1, $t0, 3	# expected result: 16
	jr	$ra
	
test_sllv:
	li  	$t2, 3
	li  	$t3, 5
	sll	$a1, $t2, $t3   # expected result: 96
	jr	$ra

test_slt_false:
	li	$s0, 1
	li	$s1, -1
	slt	$a1, $s0, $s1	# expected: 0
	jr	$ra

test_slt_true:
	li	$s0, -1
	li	$s1, 1
	slt 	$a1, $s0, $a1	# expected: 1
	jr	$ra

test_slti_false:
	li	$s0, 2345
	slti	$a1, $s0, 2345 	# expected: 0
	jr	$ra

test_slti_true:
	li	$s0, -23
	slti	$a1, $s0, 22
	jr	$ra		# expected: 1

test_sltiu_false:
	li	$s0, -22
	sltiu	$a1, $s0, 22 	# expected: 0
	jr	$ra

test_sltiu_true:
	li	$s0, 22
	sltiu	$a1, $s0, -22	# expected: 1
	jr	$ra

test_sltiu_signext:
        li      $s0, 0xffffa000
        sltiu   $a1, $s0, 0xb000   # expected: 1 (because b000 should be sign extended to ffffb000)
        jr      $ra
        
test_sltu_false:
	li	$s0, -22
	li	$s1, 22
	sltu	$a1, $s0, $s1	# expected: 0
	jr	$ra

test_sltu_true:
	li	$s0, 22
	li	$s1, -22
	sltu	$a1, $s0, $s1	# expected: 1
	jr	$ra

test_sra:
	li	$t0, -2
	sra	$a1, $t0, 1	# expected: -1
	jr 	$ra

test_srav:
	li	$t0, -8
	li	$t1, 2
	srav	$a1, $t0, $t1	# expected: -2
	jr	$ra
	
test_srl:
	li	$t0, -2
	srl	$a1, $t0, 1	# expected: 2147483647
	jr	$ra

test_srlv:
	li	$t0, -8
	li	$t1, 2
	srlv	$a1, $t0, $t1	# expected: 1073741822
	jr	$ra

test_sub:
	li	$t0, 30
	li	$t1, 20
	sub	$a1, $t0, $t1
	jr	$ra		# expected: 10

test_subu:
	li	$t0, 30
	li	$t1, 20
	subu	$a1, $t0, $t1	# expected: 10
	jr	$ra

test_xor:
	li	$t0, 0xff00
	li	$t1, 0xf0f0
	xor	$a1, $t0, $t1 	# expected: 0x0ff0 = 4080
	jr	$ra

test_xori:
	li	$t0, 0xff00
	xori	$a1, $t0, 0xf0f0
	jr	$ra		# expected: 4080


# Test that data sections are copied correctly into the executable
test_dat1:
	la	$a1, dat1
	lw	$a1, ($a1)
	jr	$ra

test_dat2:
	la	$a1, dat2
	lw	$a1, ($a1)
	jr	$ra

# Test that BSS sections are correctly zeroed out in the executable
test_bss:	
	la	$t0, bss1	# ptr
	la	$t1, bss_count	
	lw	$t1, ($t1)	# count (no of words)
	li	$a1, 0
.loop:
	lw	$t2, ($t0)
	or	$a1, $a1, $t2	# a1 = a1 or (*ptr)
	addiu	$t1, $t1, -1	# decrease counter
	bnez	$t1, .loop	# if counter is not zero yet, loop
	jr	$ra		# expected result: zero.


lwtest:
        # auxiliary routine
        # inputs:
        #   t0 = expected left mem word (after swl/swr)
        #   t1 = expected right mem word (after swl/swr)
        #   t2 = expected register after lwl
        #   t3 = expected register after lwr
        #   t4 = offset
        # temporary registers used:
        #   s0 = addr of temp space (left word)
        #   s1 = addr of temp space (right word)
        #   s2 = 'left' addr (for swl, lwl)
        #   s3 = 'right' addr (for swr, lwr)
        #   s4 = test bit pattern (0x12345678)
        #   k0 = temporary for reading back output
        # outputs:
        #   a1 = error output. will be zero if test passed.

        # setup:
        # allocate two test words. put addrs into s0 and s1. clear them to start with.
        # put 'left' addr into s2 and 'right' addr into s3.
        # put the test word (0x12345678) into t0.
        addiu   $s0, $sp, -8
        addiu   $s1, $sp, -4
        sw      $zero, ($s0)
        sw      $zero, ($s1)
        addu    $s2, $s0, $t4
        addiu   $s3, $s2, 3
        li      $s4, 0x12345678
        li      $a1, 0
        
        # do swl, and check result
        swl     $s4, ($s2)
        lw      $k0, ($s0)
        subu    $k0, $k0, $t0
        or      $a1, $a1, $k0

        # clear everything
        sw      $zero, ($s0)
        sw      $zero, ($s1)

        # do swr, and check result
        swr     $s4, ($s3)
        lw      $k0, ($s1)
        subu    $k0, $k0, $t1
        or      $a1, $a1, $k0

        # put the expected bit pattern into memory, ready for lwl/lwr test
        sw      $t0, ($s0)
        sw      $t1, ($s1)

        # lwl test
        li      $k0, 0
        lwl     $k0, ($s2)
        subu    $k0, $k0, $t2
        or      $a1, $a1, $k0

        # lwr test
        li      $k0, 0
        lwr     $k0, ($s3)
        subu    $k0, $k0, $t3
        or      $a1, $a1, $k0

        # all done!
        jr      $ra

lwtest2:
        # Test combinations of lwl, lwr.
        # Inputs:
        #   t0 = starting memory location.
        #   t1 = expected value.
        #   t2 = Used to do the load. Should be set to a random pattern by the caller.
        # Outputs:
        #   a1 = error output. zero if test passed.

        lwl     $t2, ($t0)
        lwr     $t2, 3($t0)
        subu    $a1, $t2, $t1    # should be zero
        jr      $ra
        

test_lw_0:
        li      $t0, 0x12345678
        li      $t1, 0
        li      $t2, 0x12345678
        li      $t3, 0x12345678
        li      $t4, 0
        j       lwtest

test_lw_1:
        li      $t0, 0x00123456
        li      $t1, 0x78000000
        li      $t2, 0x12345600
        li      $t3, 0x00000078
        li      $t4, 1
        j       lwtest

test_lw_2:
        li      $t0, 0x00001234
        li      $t1, 0x56780000
        li      $t2, 0x12340000
        li      $t3, 0x00005678
        li      $t4, 2
        j       lwtest

test_lw_3:
        li      $t0, 0x00000012
        li      $t1, 0x34567800
        li      $t2, 0x12000000
        li      $t3, 0x00345678
        li      $t4, 3
        j       lwtest

test_lw_4:
        li      $t0, 0
        li      $t1, 0x12345678
        li      $t2, 0x12345678
        li      $t3, 0x12345678
        li      $t4, 4
        j       lwtest

test_lw_new_0:
        la      $k0, lwtest2
        la      $t0, lwbytes0
        li      $t1, 0x12345678
        li      $t2, 0x55555555
        jr      $k0     # call via register to prevent certain Mips2cs optimizations.

test_lw_new_1:
        la      $k0, lwtest2
        la      $t0, lwbytes1
        li      $t1, 0x5432abcd
        li      $t2, 0xaaaaaaaa
        jr      $k0

test_lw_new_2:
        la      $k0, lwtest2
        la      $t0, lwbytes2
        li      $t1, 0xfeaf1124
        li      $t2, 0x11111111
        jr      $k0

test_lw_new_3:
        la      $k0, lwtest2
        la      $t0, lwbytes3
        li      $t1, 0x65430012
        li      $t2, 0xffddc761
        jr      $k0

test_mtlo:
        li      $s2, 42
        mtlo    $s2
        mflo    $a1       # expected result: 42
        jr      $ra
        
test_mthi:
        li      $s1, 4
        mthi    $s1
        mfhi    $a1       # expected result: 4
        jr      $ra

run_all_tests:
	# This routine lists all the tests to be called (and their names)

	# save ra to the stack
	addiu	$sp, $sp, -4
	sw	$ra, ($sp)

	
	#### Run all the tests ####

	la	$a0, msg_add
	la	$a1, test_add
	jal	dotest

	la	$a0, msg_addi
	la	$a1, test_addi
	jal	dotest

	la	$a0, msg_addiu
	la	$a1, test_addiu
	jal	dotest

        la      $a0, msg_addiu_neg
        la      $a1, test_addiu_neg
        jal     dotest
        
	la	$a0, msg_addu
	la	$a1, test_addu
	jal	dotest

	la	$a0, msg_and
	la	$a1, test_and
	jal	dotest

	la	$a0, msg_andi
	la	$a1, test_andi
	jal	dotest

        la      $a0, msg_andi_neg
        la      $a1, test_andi_neg
        jal     dotest
        
	la	$a0, msg_beq_false
	la	$a1, test_beq_false
	jal	dotest

	la	$a0, msg_beq_true
	la	$a1, test_beq_true
	jal	dotest

	la	$a0, msg_bgez_false
	la	$a1, test_bgez_false
	jal	dotest

	la	$a0, msg_bgez_true
	la	$a1, test_bgez_true
	jal	dotest

	la	$a0, msg_bgezal_false
	la	$a1, test_bgezal_false
	jal	dotest

	la	$a0, msg_bgezal_true
	la	$a1, test_bgezal_true
	jal	dotest

        la      $a0, msg_bgezal_ra
        la      $a1, test_bgezal_ra
        jal     dotest

	la	$a0, msg_bgtz_false
	la	$a1, test_bgtz_false
	jal	dotest

	la 	$a0, msg_bgtz_true
	la	$a1, test_bgtz_true
	jal	dotest

	la	$a0, msg_blez_false
	la	$a1, test_blez_false
	jal	dotest

	la	$a0, msg_blez_true
	la	$a1, test_blez_true
	jal	dotest

	la	$a0, msg_bltz_false
	la	$a1, test_bltz_false
	jal	dotest

	la	$a0, msg_bltz_true
	la	$a1, test_bltz_true
	jal	dotest

	la	$a0, msg_bltzal_false
	la 	$a1, test_bltzal_false
	jal	dotest

	la	$a0, msg_bltzal_true
	la	$a1, test_bltzal_true
	jal	dotest

        la      $a0, msg_bltzal_ra
        la      $a1, test_bltzal_ra
        jal     dotest
        
	la	$a0, msg_bne_false
	la	$a1, test_bne_false
	jal	dotest

	la	$a0, msg_bne_true
	la	$a1, test_bne_true
	jal	dotest

        la      $a0, msg_delay
        la      $a1, test_delay
        jal     dotest
        
	la	$a0, msg_div_hi
	la	$a1, test_div_hi
	jal	dotest

	la	$a0, msg_div_lo
	la	$a1, test_div_lo
	jal	dotest

	la	$a0, msg_div_2hi
	la	$a1, test_div_2hi
	jal	dotest

	la	$a0, msg_div_2lo
	la	$a1, test_div_2lo
	jal	dotest

	la 	$a0, msg_divu_hi
	la	$a1, test_divu_hi
	jal	dotest

	la	$a0, msg_divu_lo
	la	$a1, test_divu_lo
	jal	dotest

	la	$a0, msg_divu_2hi
	la	$a1, test_divu_2hi
	jal	dotest

	la	$a0, msg_divu_2lo
	la	$a1, test_divu_2lo
	jal	dotest

	la	$a0, msg_sb_lb
	la	$a1, test_sb_lb
	jal	dotest

	la	$a0, msg_sb_lb_2
	la	$a1, test_sb_lb_2
	jal	dotest

	la	$a0, msg_sb_lbu
	la	$a1, test_sb_lbu
	jal	dotest
	
	la	$a0, msg_sh_lh
	la	$a1, test_sh_lh
	jal	dotest

	la	$a0, msg_sh_lh_2
	la	$a1, test_sh_lh_2
	jal	dotest

	la	$a0, msg_sh_lhu
	la	$a1, test_sh_lhu
	jal	dotest
	
	la	$a0, msg_lui
	la	$a1, test_lui
	jal	dotest

	la	$a0, msg_sw_lw
	la	$a1, test_sw_lw
	jal	dotest

	la	$a0, msg_mult_hi
	la	$a1, test_mult_hi
	jal	dotest

	la	$a0, msg_mult_lo
	la	$a1, test_mult_lo
	jal	dotest

	la	$a0, msg_mult_2hi
	la	$a1, test_mult_2hi
	jal	dotest

	la	$a0, msg_mult_2lo
	la	$a1, test_mult_2lo
	jal	dotest

	la	$a0, msg_multu_hi
	la	$a1, test_multu_hi
	jal	dotest

	la 	$a0, msg_multu_lo
	la	$a1, test_multu_lo
	jal	dotest

	la	$a0, msg_multu_2hi
	la	$a1, test_multu_2hi
	jal	dotest

	la	$a0, msg_multu_2lo
	la	$a1, test_multu_2lo
	jal	dotest

	la	$a0, msg_or
	la	$a1, test_or
	jal	dotest

	la	$a0, msg_ori
	la	$a1, test_ori
	jal	dotest

	la  	$a0, msg_sll
	la  	$a1, test_sll
	jal	dotest

	la  	$a0, msg_sllv
	la  	$a1, test_sllv
	jal	dotest

	la	$a0, msg_slt_false
	la	$a1, test_slt_false
	jal	dotest

	la	$a0, msg_slt_true
	la	$a1, test_slt_true
	jal 	dotest

	la	$a0, msg_slti_false
	la	$a1, test_slti_false
	jal	dotest

	la	$a0, msg_slti_true
	la	$a1, test_slti_true
	jal	dotest

	la	$a0, msg_sltiu_false
	la	$a1, test_sltiu_false
	jal	dotest

	la	$a0, msg_sltiu_true
	la	$a1, test_sltiu_true
	jal	dotest

        la      $a0, msg_sltiu_signext
        la      $a1, test_sltiu_signext
        jal     dotest

	la 	$a0, msg_sltu_false
	la	$a1, test_sltu_false
	jal	dotest

	la	$a0, msg_sltu_true
	la	$a1, test_sltu_true
	jal	dotest

	la	$a0, msg_sra
	la	$a1, test_sra
	jal	dotest

	la	$a0, msg_srav
	la	$a1, test_srav
	jal	dotest

	la	$a0, msg_srl
	la	$a1, test_srl
	jal	dotest

	la	$a0, msg_srlv
	la	$a1, test_srlv
	jal	dotest

	la	$a0, msg_sub
	la	$a1, test_sub
	jal	dotest

	la	$a0, msg_subu
	la	$a1, test_subu
	jal	dotest

	la	$a0, msg_xor
	la	$a1, test_xor
	jal	dotest

	la	$a0, msg_xori
	la	$a1, test_xori
	jal	dotest

	la	$a0, msg_dat1
	la	$a1, test_dat1
	jal	dotest

	la	$a0, msg_dat2
	la	$a1, test_dat2
	jal	dotest

	la	$a0, msg_bss
	la	$a1, test_bss
	jal	dotest

        la      $a0, msg_lw_0
        la      $a1, test_lw_0
        jal     dotest

        la      $a0, msg_lw_1
        la      $a1, test_lw_1
        jal     dotest

        la      $a0, msg_lw_2
        la      $a1, test_lw_2
        jal     dotest

        la      $a0, msg_lw_3
        la      $a1, test_lw_3
        jal     dotest

        la      $a0, msg_lw_4
        la      $a1, test_lw_4
        jal     dotest

        la      $a0, msg_lw_new_0
        la      $a1, test_lw_new_0
        jal     dotest

        la      $a0, msg_lw_new_1
        la      $a1, test_lw_new_1
        jal     dotest

        la      $a0, msg_lw_new_2
        la      $a1, test_lw_new_2
        jal     dotest

        la      $a0, msg_lw_new_3
        la      $a1, test_lw_new_3
        jal     dotest

        la      $a0, msg_mtlo
        la      $a1, test_mtlo
        jal     dotest

        la      $a0, msg_mthi
        la      $a1, test_mthi
        jal     dotest

	# Restore ra and return
	lw	$ra, ($sp)
	addiu	$sp, $sp, 4
	jr	$ra

	
	
	.data

	.align 2

	# Names for the test cases
msg_add:	.asciz "add"
msg_addi:	.asciz "addi"
msg_addiu:	.asciz "addiu"
msg_addiu_neg:  .asciz "addiu_neg"
msg_addu:	.asciz "addu"
msg_and:	.asciz "and"
msg_andi:	.asciz "andi"
msg_andi_neg:   .asciz "andi_neg"
msg_beq_false:	.asciz "beq_false"
msg_beq_true:	.asciz "beq_true"
msg_bgez_false:	.asciz "bgez_false"
msg_bgez_true:	.asciz "bgez_true"
msg_bgezal_false:	.asciz "bgezal_false"
msg_bgezal_true:	.asciz "bgezal_true"
msg_bgezal_ra:  .asciz "bgezal_ra"
msg_bgtz_false:	.asciz "bgtz_false"
msg_bgtz_true:	.asciz "bgtz_true"
msg_blez_false:	.asciz "blez_false"
msg_blez_true:	.asciz "blez_true"
msg_bltz_false:	.asciz "bltz_false"
msg_bltz_true:	.asciz "bltz_true"
msg_bltzal_false:	.asciz "bltzal_false"
msg_bltzal_true:	.asciz "bltzal_true"
msg_bltzal_ra:  .asciz "bltzal_ra"
msg_bne_false:	.asciz "bne_false"
msg_bne_true:	.asciz "bne_true"
msg_delay:      .asciz "delay"
msg_div_hi:	.asciz "div_hi"
msg_div_lo:	.asciz "div_lo"
msg_div_2hi:	.asciz "div_2hi"
msg_div_2lo:	.asciz "div_2lo"
msg_divu_hi:	.asciz "divu_hi"
msg_divu_lo:	.asciz "divu_lo"
msg_divu_2hi:	.asciz "divu_2hi"
msg_divu_2lo:	.asciz "divu_2lo"
msg_sb_lb:	.asciz "sb_lb"
msg_sb_lb_2:	.asciz "sb_lb_2"
msg_sb_lbu:	.asciz "sb_lbu"
msg_sh_lh:	.asciz "sh_lh"
msg_sh_lh_2:	.asciz "sh_lh_2"
msg_sh_lhu:	.asciz "sh_lhu"
msg_lui:	.asciz "lui"
msg_sw_lw:	.asciz "sw_lw"
msg_mult_hi:	.asciz "mult_hi"
msg_mult_lo:	.asciz "mult_lo"
msg_mult_2hi:	.asciz "mult_2hi"
msg_mult_2lo:	.asciz "mult_2lo"
msg_multu_hi:	.asciz "multu_hi"
msg_multu_lo:	.asciz "multu_lo"
msg_multu_2hi:	.asciz "multu_2hi"
msg_multu_2lo:	.asciz "multu_2lo"
msg_or:		.asciz "or"
msg_ori:	.asciz "ori"
msg_sll:	.asciz "sll"
msg_sllv:	.asciz "sllv"
msg_slt_false:	.asciz "slt_false"
msg_slt_true:	.asciz "slt_true"
msg_slti_false:	.asciz "slti_false"
msg_slti_true:	.asciz "slti_true"
msg_sltiu_false:	.asciz "sltiu_false"
msg_sltiu_true:	.asciz "sltiu_true"
msg_sltiu_signext:      .asciz "sltiu_signext"
msg_sltu_false:	.asciz "sltu_false"
msg_sltu_true:	.asciz "sltu_true"
msg_sra:	.asciz "sra"
msg_srav:	.asciz "srav"
msg_srl:	.asciz "srl"
msg_srlv:	.asciz "srlv"
msg_sub:	.asciz "sub"
msg_subu:	.asciz "subu"
msg_xor:	.asciz "xor"
msg_xori:	.asciz "xori"
msg_dat1:	.asciz "dat1"
msg_dat2:	.asciz "dat2"
msg_bss:	.asciz "bss"
msg_lw_0:       .asciz "lw_0"
msg_lw_1:       .asciz "lw_1"
msg_lw_2:       .asciz "lw_2"
msg_lw_3:       .asciz "lw_3"
msg_lw_4:       .asciz "lw_4"
msg_lw_new_0:   .asciz "lw_new_0"
msg_lw_new_1:   .asciz "lw_new_1"
msg_lw_new_2:   .asciz "lw_new_2"
msg_lw_new_3:   .asciz "lw_new_3"
msg_mtlo:       .asciz "mtlo"
msg_mthi:       .asciz "mthi"
        

        .align 2
lwbytes0:       .byte 0x12, 0x34, 0x56, 0x78, 0
lwbytes1:       .byte 0x54, 0x32, 0xab, 0xcd, 0
lwbytes2:       .byte 0xfe, 0xaf, 0x11, 0x24, 0
lwbytes3:       .byte 0x65, 0x43, 0x00, 0x12, 0
        

	.section "custom_data_1", "a"
dat1:	.word 12345

	.section "custom_data_2", "a"
dat2:	.word 67890
bss_count:	.word 500
	
	.bss
bss1:	.space 2000
msg_buf:	.space 1024
num_buf:	.space 32
	
	.text

dotest:
	# Driver routine to run a test.
	# Input: test name in $a0 and pointer to test code in $a1.
	# We clear all other registers then call the test routine.
	# The test results are reported by calling address 16 with a0=message, a1=length.
	
	# save the return address
	addiu	$sp, $sp, -4
	sw	$ra, ($sp)

	# clear all registers (except a0 and a1 which are input arguments, and sp and ra.)
	li  	$at, 0
	li  	$v0, 0
	li  	$v1, 0
	li  	$a2, 0
	li  	$a3, 0
	li  	$t0, 0
	li  	$t1, 0
	li  	$t2, 0
	li  	$t3, 0
	li  	$t4, 0
	li  	$t5, 0
	li  	$t6, 0
	li  	$t7, 0
	li  	$t8, 0
	li  	$t9, 0
	li  	$s0, 0
	li  	$s1, 0
	li  	$s2, 0
	li  	$s3, 0
	li  	$s4, 0
	li  	$s5, 0
	li  	$s6, 0
	li  	$s7, 0
	li  	$s8, 0
	li	$k0, 0
	li	$k1, 0
	li  	$gp, 0
	
	# call the test case
	jalr	$a1

	# PRINT the test result
	# a0 = test name ptr, a1 = result (integer)
	# we call addr 16 with a0 = ptr to msg, a1 = length.

	# point $s0 at the msg buffer
	la	$s0, msg_buf

copy_loop:	
	# copy testcase name from $a0 to the msg buffer ($s0)
	lb	$t0, ($a0)
	beq	$t0, $zero, exit_copy_loop
	sb	$t0, ($s0)
	addiu	$a0, $a0, 1
	addiu	$s0, $s0, 1
	j	copy_loop

exit_copy_loop:	

	# now print ": "
	li	$t0, ':'
	sb	$t0, ($s0)
	li	$t0, ' '
	sb	$t0, 1($s0)
	addiu	$s0, $s0, 2
	
	# now write $a1 as number to num_buf...
	
	# point $s1 at num_buf
	la	$s1, num_buf

	# special case if $a1 is zero
	bne	$a1, $zero, num_not_zero
	li	$t0, '0'
	sb	$t0, ($s1)
	addiu	$s1, $s1, 1
	j	flip_loop

num_not_zero:	
	# special case if $a1 < 0
	bgt	$a1, $zero, num_loop
	li	$t0, '-'
	sb	$t0, ($s0)
	addiu	$s0, $s0, 1
	subu	$a1, $zero, $a1

	# now we keep dividing by 10 until the number is nonzero
num_loop:
	li	$t0, 10
	divu	$zero, $a1, $t0
	# hi = remainder
	# lo = quotient
	mfhi	$t0
	mflo	$a1
	addiu	$t0, $t0, '0'
	sb	$t0, ($s1)
	addiu	$s1, $s1, 1
	bne	$a1, $zero, num_loop

	# now flip the order of the num_buf and write it to $s0
flip_loop:
	addiu	$s1, $s1, -1
	lb	$t0, ($s1)
	sb	$t0, ($s0)
	addiu	$s0, $s0, 1
	la	$t0, num_buf
	subu	$t0, $s1, $t0   # $t0 = $s1 - num_buf
	bgtz	$t0, flip_loop  # jump if $s1 > num_buf

	# store final newline character
	li	$t0, '\n'
	sb	$t0, ($s0)
	addiu	$s0, $s0, 1
	
	# now call the output routine.
	la	$t0, msg_buf
	subu	$a1, $s0, $t0
	la	$a0, msg_buf
	jal	16

	# now get ra back off the stack and return.
	lw	$ra, ($sp)
	addiu	$sp, $sp, 4
	jr	$ra

	
	
_start:	
.global _start
	# The start routine.
	jal	run_all_tests
	j	0		# Magic exit address.
